==> diamond-aligner_0.9.29+dfsg.orig.tar.xz: diamond-0.9.29/src/util/data_structures/double_array.h <==
#ifndef DOUBLE_ARRAY_H_
#define DOUBLE_ARRAY_H_

#include <stdint.h>
#include <stddef.h>
#include <algorithm>
#include "../range.h"

template<typename _t>
struct DoubleArray {

	DoubleArray()
	{}

	DoubleArray(void* data, size_t size = 0):
		data_((char*)data),
		size_(size)
	{}

	struct Iterator {

		Iterator(void *ptr) :
			ptr_((char*)ptr),
			end_(nullptr)
		{}

		Iterator(char *ptr, char *end) :
			ptr_(ptr),
			end_(end)
		{
			skip_del();
		}

		uint32_t& count() {
			return *(uint32_t*)ptr_;
		}

		Range<_t*> operator*() {
			return { (_t*)(ptr_ + 4), (_t*)(ptr_ + 4) + count() };
		}

		Range<_t*>* operator->() {
			range_ = this->operator*();
			return &range_;
		}

		Iterator& operator++() {
			next();
			skip_del();
			return *this;
		}

		void next() {
			ptr_ += count() * sizeof(_t) + 4;
		}

		operator bool() const {
			return ptr_ < end_;
		}

		void erase() {
			uint32_t n = count();
			*(uint32_t*)(ptr_ + 4) = n;
			count() = 0;
			ptr_ += n * sizeof(_t) + 4;
		}

		ptrdiff_t operator-(const Iterator &x) const {
			return ptr_ - x.ptr_;
		}

		bool operator==(const Iterator &x) const {
			return ptr_ == x.ptr_;
		}

	private:

		void skip_del() {
			while (ptr_ < end_ && count() == 0)
				ptr_ += (*(uint32_t*)(ptr_ + 4)) * sizeof(_t) + 4;
		}

		Range<_t*> range_;
		char *ptr_, *end_;

		friend struct DoubleArray;

	};

	Iterator begin() {
		return Iterator(data_, data_ + size_);
	}

	void set_end(const Iterator &it) {
		size_ = it.ptr_ - data_;
	}

	void append(const DoubleArray &d) {
		if (d.data_ == data_ + size_) {
			size_ += d.size_;
			return;
		}
		std::copy(d.data_, d.data_ + d.size_, data_ + size_);
		size_ += d.size_;
	}

	uint32_t offset(const Iterator &it) const {
		return uint32_t(it.ptr_ - data_);
	}

	_t& operator[](uint32_t i) {
		return *(_t*)(data_ + i);
	}

private:

	char *data_;
	size_t size_;

};

#endif

==> diamond-aligner_0.9.30.orig.tar.xz: diamond-0.9.30/src/util/data_structures/double_array.h <==
/****
DIAMOND protein aligner
Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
                        Benjamin Buchfink
                        Eberhard Karls Universitaet Tuebingen

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
****/

#ifndef DOUBLE_ARRAY_H_
#define DOUBLE_ARRAY_H_

#include <stdint.h>
#include <stddef.h>
#include <algorithm>
#include "../range.h"

template<typename _t>
struct DoubleArray {

	DoubleArray()
	{}

	DoubleArray(void* data, size_t size = 0):
		data_((char*)data),
		size_(size)
	{}

	struct Iterator {

		Iterator(void *ptr) :
			ptr_((char*)ptr),
			end_(nullptr)
		{}

		Iterator(char *ptr, char *end) :
			ptr_(ptr),
			end_(end)
		{
			skip_del();
		}

		uint32_t& count() {
			return *(uint32_t*)ptr_;
		}

		Range<_t*> operator*() {
			return { (_t*)(ptr_ + 4), (_t*)(ptr_ + 4) + count() };
		}

		Range<_t*>* operator->() {
			range_ = this->operator*();
			return &range_;
		}

		Iterator& operator++() {
			next();
			skip_del();
			return *this;
		}

		void next() {
			ptr_ += count() * sizeof(_t) + 4;
		}

		operator bool() const {
			return ptr_ < end_;
		}

		void erase() {
			uint32_t n = count();
			*(uint32_t*)(ptr_ + 4) = n;
			count() = 0;
			ptr_ += n * sizeof(_t) + 4;
		}

		ptrdiff_t operator-(const Iterator &x) const {
			return ptr_ - x.ptr_;
		}

		bool operator==(const Iterator &x) const {
			return ptr_ == x.ptr_;
		}

	private:

		void skip_del() {
			while (ptr_ < end_ && count() == 0)
				ptr_ += (*(uint32_t*)(ptr_ + 4)) * sizeof(_t) + 4;
		}

		Range<_t*> range_;
		char *ptr_, *end_;

		friend struct DoubleArray;

	};

	Iterator begin() {
		return Iterator(data_, data_ + size_);
	}

	void set_end(const Iterator &it) {
		size_ = it.ptr_ - data_;
	}

	void append(const DoubleArray &d) {
		if (d.data_ == data_ + size_) {
			size_ += d.size_;
			return;
		}
		std::copy(d.data_, d.data_ + d.size_, data_ + size_);
		size_ += d.size_;
	}

	uint32_t offset(const Iterator &it) const {
		return uint32_t(it.ptr_ - data_);
	}

	_t& operator[](uint32_t i) {
		return *(_t*)(data_ + i);
	}

private:

	char *data_;
	size_t size_;

};

#endif
